-- Test contact
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(90);

set search_path to numerus, auth, public;

select has_table('contact');
select has_pk('contact' );
select table_privs_are('contact', 'guest', array []::text[]);
select table_privs_are('contact', 'invoicer', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('contact', 'admin', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('contact', 'authenticator', array []::text[]);

select has_sequence('contact_contact_id_seq');
select sequence_privs_are('contact_contact_id_seq', 'guest', array[]::text[]);
select sequence_privs_are('contact_contact_id_seq', 'invoicer', array['USAGE']);
select sequence_privs_are('contact_contact_id_seq', 'admin', array['USAGE']);
select sequence_privs_are('contact_contact_id_seq', 'authenticator', array[]::text[]);

select has_column('contact', 'contact_id');
select col_is_pk('contact', 'contact_id');
select col_type_is('contact', 'contact_id', 'integer');
select col_not_null('contact', 'contact_id');
select col_has_default('contact', 'contact_id');
select col_default_is('contact', 'contact_id', 'nextval(''contact_contact_id_seq''::regclass)');

select has_column('contact', 'company_id');
select col_is_fk('contact', 'company_id');
select fk_ok('contact', 'company_id', 'company', 'company_id');
select col_type_is('contact', 'company_id', 'integer');
select col_not_null('contact', 'company_id');
select col_hasnt_default('contact', 'company_id');

select has_column('contact', 'slug');
select col_is_unique('contact', 'slug');
select col_type_is('contact', 'slug', 'uuid');
select col_not_null('contact', 'slug');
select col_has_default('contact', 'slug');
select col_default_is('contact', 'slug', 'gen_random_uuid()');

select has_column('contact', 'business_name');
select col_type_is('contact', 'business_name', 'text');
select col_not_null('contact', 'business_name');
select col_hasnt_default('contact', 'business_name');

select has_column('contact', 'vatin');
select col_type_is('contact', 'vatin', 'vatin');
select col_not_null('contact', 'vatin');
select col_hasnt_default('contact', 'vatin');

select has_column('contact', 'trade_name');
select col_type_is('contact', 'trade_name', 'text');
select col_not_null('contact', 'trade_name');
select col_hasnt_default('contact', 'trade_name');

select has_column('contact', 'phone');
select col_type_is('contact', 'phone', 'packed_phone_number');
select col_not_null('contact', 'phone');
select col_hasnt_default('contact', 'phone');

select has_column('contact', 'email');
select col_type_is('contact', 'email', 'email');
select col_not_null('contact', 'email');
select col_hasnt_default('contact', 'email');

select has_column('contact', 'web');
select col_type_is('contact', 'web', 'uri');
select col_not_null('contact', 'web');
select col_hasnt_default('contact', 'web');

select has_column('contact', 'address');
select col_type_is('contact', 'address', 'text');
select col_not_null('contact', 'address');
select col_hasnt_default('contact', 'address');

select has_column('contact', 'city');
select col_type_is('contact', 'city', 'text');
select col_not_null('contact', 'city');
select col_hasnt_default('contact', 'city');

select has_column('contact', 'province');
select col_type_is('contact', 'province', 'text');
select col_not_null('contact', 'province');
select col_hasnt_default('contact', 'province');

select has_column('contact', 'postal_code');
select col_type_is('contact', 'postal_code', 'text');
select col_not_null('contact', 'postal_code');
select col_hasnt_default('contact', 'postal_code');

select has_column('contact', 'country_code');
select col_type_is('contact', 'country_code', 'country_code');
select col_is_fk('contact', 'country_code');
select col_type_is('contact', 'country_code', 'country_code');
select col_not_null('contact', 'country_code');
select col_hasnt_default('contact', 'country_code');

select has_column('contact', 'tags');
select col_type_is('contact', 'tags', 'tag_name[]');
select col_not_null('contact', 'tags');
select col_has_default('contact', 'tags');
select col_default_is('contact', 'tags', '{}');

select has_column('contact', 'created_at');
select col_type_is('contact', 'created_at', 'timestamp with time zone');
select col_not_null('contact', 'created_at');
select col_has_default('contact', 'created_at');
select col_default_is('contact', 'created_at', current_timestamp);

set client_min_messages to warning;
truncate contact cascade;
truncate company_user cascade;
truncate payment_method cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

set constraints "company_default_payment_method_id_fkey" deferred;

insert into auth."user" (user_id, email, name, password, role, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', 'invoicer', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', 'admin', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_payment_method_id)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 222)
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', 'FR', 'USD', 444)
;

insert into payment_method (payment_method_id, company_id, name, instructions)
values (444, 4, 'cash', 'cash')
     , (222, 2, 'cash', 'cash')
;

set constraints "company_default_payment_method_id_fkey" immediate;

insert into company_user (company_id, user_id)
values (2, 1)
     , (4, 5)
;

insert into contact (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code)
values (2, 'Contact 1', 'XX555', '', '777-777-777', 'c@c', '', '', '', '', '', 'ES')
     , (4, 'Contact 2', 'XX666', '', '888-888-888', 'd@d', '', '', '', '', '', 'ES')
;

prepare contact_data as
select company_id, business_name
from contact
order by company_id, business_name;

set role invoicer;
select is_empty('contact_data', 'Should show no data when cookie is not set yet');
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog');
select bag_eq(
	'contact_data',
	$$ values (2, 'Contact 1')
	$$,
	'Should only list contacts of the companies where demo@tandem.blog is user of'
);
reset role;

select set_cookie('12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog');
select bag_eq(
	'contact_data',
	$$ values (4, 'Contact 2')
	$$,
	'Should only list contacts of the companies where admin@tandem.blog is user of'
);
reset role;

select set_cookie('not-a-cookie');
select throws_ok(
	'contact_data',
	'42501', 'permission denied for table contact',
	'Should not allow select to guest users'
);
reset role;

select throws_ok( $$
	insert into contact (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code)
	values (2, '   ', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES')
	$$,
       '23514', 'new row for relation "contact" violates check constraint "business_name_not_empty"',
	'Should not allow contacts with blank business name'
);


select *
from finish();

rollback;

