-- Test invoice_product
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(51);

set search_path to numerus, auth, public;

select has_table('invoice_product');
select has_pk('invoice_product' );
select table_privs_are('invoice_product', 'guest', array []::text[]);
select table_privs_are('invoice_product', 'invoicer', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('invoice_product', 'admin', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('invoice_product', 'authenticator', array []::text[]);

select has_sequence('invoice_product_invoice_product_id_seq');
select sequence_privs_are('invoice_product_invoice_product_id_seq', 'guest', array[]::text[]);
select sequence_privs_are('invoice_product_invoice_product_id_seq', 'invoicer', array['USAGE']);
select sequence_privs_are('invoice_product_invoice_product_id_seq', 'admin', array['USAGE']);
select sequence_privs_are('invoice_product_invoice_product_id_seq', 'authenticator', array[]::text[]);

select has_column('invoice_product', 'invoice_product_id');
select col_is_pk('invoice_product', 'invoice_product_id');
select col_type_is('invoice_product', 'invoice_product_id', 'integer');
select col_not_null('invoice_product', 'invoice_product_id');
select col_has_default('invoice_product', 'invoice_product_id');
select col_default_is('invoice_product', 'invoice_product_id', 'nextval(''invoice_product_invoice_product_id_seq''::regclass)');

select has_column('invoice_product', 'invoice_id');
select col_is_fk('invoice_product', 'invoice_id');
select fk_ok('invoice_product', 'invoice_id', 'invoice', 'invoice_id');
select col_type_is('invoice_product', 'invoice_id', 'integer');
select col_not_null('invoice_product', 'invoice_id');
select col_hasnt_default('invoice_product', 'invoice_id');

select has_column('invoice_product', 'name');
select col_type_is('invoice_product', 'name', 'text');
select col_not_null('invoice_product', 'name');
select col_hasnt_default('invoice_product', 'name');

select has_column('invoice_product', 'description');
select col_type_is('invoice_product', 'description', 'text');
select col_not_null('invoice_product', 'description');
select col_has_default('invoice_product', 'description');
select col_default_is('invoice_product', 'description', '');

select has_column('invoice_product', 'price');
select col_type_is('invoice_product', 'price', 'integer');
select col_not_null('invoice_product', 'price');
select col_hasnt_default('invoice_product', 'price');

select has_column('invoice_product', 'quantity');
select col_type_is('invoice_product', 'quantity', 'integer');
select col_not_null('invoice_product', 'quantity');
select col_has_default('invoice_product', 'quantity');
select col_default_is('invoice_product', 'quantity', 1);

select has_column('invoice_product', 'discount_rate');
select col_type_is('invoice_product', 'discount_rate', 'discount_rate');
select col_not_null('invoice_product', 'discount_rate');
select col_has_default('invoice_product', 'discount_rate');
select col_default_is('invoice_product', 'discount_rate', '0.0');


set client_min_messages to warning;
truncate invoice_product cascade;
truncate invoice cascade;
truncate contact cascade;
truncate company_user cascade;
truncate payment_method cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, role, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', 'invoicer', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', 'admin', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

set constraints "company_default_payment_method_id_fkey" deferred;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_payment_method_id)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 222)
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', 'FR', 'USD', 444)
;

insert into payment_method (payment_method_id, company_id, name, instructions)
values (444, 4, 'cash', 'cash')
     , (222, 2, 'cash', 'cash')
;

set constraints "company_default_payment_method_id_fkey" immediate;

insert into company_user (company_id, user_id)
values (2, 1)
     , (4, 5)
;

insert into contact (contact_id, company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code)
values (6, 2, 'Contact 1', 'XX555', '', '777-777-777', 'c@c', '', '', '', '', '', 'ES')
     , (8, 4, 'Contact 2', 'XX666', '', '888-888-888', 'd@d', '', '', '', '', '', 'ES')
;

insert into invoice (invoice_id, company_id, invoice_number, contact_id, currency_code, payment_method_id)
values (10, 2, 'INV020001', 6, 'EUR', 222)
     , (12, 4, 'INV040001', 8, 'EUR', 444)
;

insert into invoice_product (invoice_id, name, description, price, quantity)
values (10, 'product 1', 'description 1', 1212, 1)
     , (12, 'product 2', 'description 2', 2424, 2)
;


prepare invoice_product_data as
select invoice_id, name, price, quantity
from invoice_product
order by invoice_id;

set role invoicer;
select is_empty('invoice_product_data', 'Should show no data when cookie is not set yet');
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog');
select bag_eq(
	'invoice_product_data',
	$$ values (10, 'product 1', 1212, 1)
	$$,
	'Should only list products of invoices of the companies where demo@tandem.blog is user of'
);
reset role;

select set_cookie('12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog');
select bag_eq(
	'invoice_product_data',
	$$ values (12, 'product 2', 2424, 2)
	$$,
	'Should only list products of invoices of the companies where admin@tandem.blog is user of'
);
reset role;

select set_cookie('not-a-cookie');
select throws_ok(
	'invoice_product_data',
	'42501', 'permission denied for table invoice_product',
	'Should not allow select to guest users'
);
reset role;


select throws_ok( $$
	insert into invoice_product (invoice_id, name, description, price, quantity)
	values (10, '    ', '', 1212, 1)
	$$,
	'23514', 'new row for relation "invoice_product" violates check constraint "name_not_empty"',
	'Should not allow invoice products with blank name'
);


select *
from finish();

rollback;

