-- Test product_tax
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(23);

set search_path to numerus, auth, public;

select has_table('product_tax');
select has_pk('product_tax' );
select col_is_pk('product_tax', array['product_id', 'tax_id']);
select table_privs_are('product_tax', 'guest', array []::text[]);
select table_privs_are('product_tax', 'invoicer', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('product_tax', 'admin', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('product_tax', 'authenticator', array []::text[]);

select has_column('product_tax', 'product_id');
select col_is_fk('product_tax', 'product_id');
select fk_ok('product_tax', 'product_id', 'product', 'product_id');
select col_type_is('product_tax', 'product_id', 'integer');
select col_not_null('product_tax', 'product_id');
select col_hasnt_default('product_tax', 'product_id');

select has_column('product_tax', 'tax_id');
select col_is_fk('product_tax', 'tax_id');
select fk_ok('product_tax', 'tax_id', 'tax', 'tax_id');
select col_type_is('product_tax', 'tax_id', 'integer');
select col_not_null('product_tax', 'tax_id');
select col_hasnt_default('product_tax', 'tax_id');


set client_min_messages to warning;
truncate product_tax cascade;
truncate product cascade;
truncate tax cascade;
truncate tax_class cascade;
truncate company_user cascade;
truncate payment_method cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, role, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', 'invoicer', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', 'admin', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

set constraints "company_default_payment_method_id_fkey" deferred;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_payment_method_id)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 222)
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', 'FR', 'USD', 444)
;

insert into payment_method (payment_method_id, company_id, name, instructions)
values (444, 4, 'cash', 'cash')
     , (222, 2, 'cash', 'cash')
;

set constraints "company_default_payment_method_id_fkey" immediate;

insert into company_user (company_id, user_id)
values (2, 1)
     , (4, 5)
;

insert into tax_class (tax_class_id, company_id, name)
values (22, 2, 'iva')
     , (44, 4, 'iva')
;

insert into tax (tax_id, company_id, tax_class_id, name, rate)
values (3, 2, 22, 'IVA 21 %', 0.21)
     , (6, 4, 44, 'IVA 10 %', 0.10)
;

insert into product (product_id, company_id, name, description, price)
values (7, 2, 'Product 1', 'Description 1', 1200)
     , (8, 4, 'Product 2', 'Description 2', 2400)
;

insert into product_tax (product_id, tax_id)
values (7, 3)
     , (8, 6)
;

prepare product_tax_data as
select product_id, tax_id
from product_tax
order by product_id, tax_id;

set role invoicer;
select is_empty('product_tax_data', 'Should show no data when cookie is not set yet');
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog');
select bag_eq(
	'product_tax_data',
	$$ values (7, 3)
	$$,
	'Should only list tax of products of the companies where demo@tandem.blog is user of'
);
reset role;

select set_cookie('12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog');
select bag_eq(
	'product_tax_data',
	$$ values (8, 6)
	$$,
	'Should only list tax of products of the companies where admin@tandem.blog is user of'
);
reset role;

select set_cookie('not-a-cookie');
select throws_ok(
	'product_tax_data',
	'42501', 'permission denied for table product_tax',
	'Should not allow select to guest users'
);
reset role;


select *
from finish();

rollback;

